@@ -21,9 +21,11 @@ from pyecharts.options import (
     TooltipOpts,
     VisualMapOpts,
 )
+from numpy import sqrt
 from scipy import optimize
 
 from xalpha import __path__
+from .exceptions import HttpStatusError
 
 logger = logging.getLogger(__name__)
 
@@ -54,7 +56,11 @@ yesterdaydash = lambda: dt.datetime.strftime(yesterdayobj(), "%Y-%m-%d")
 
 
 # list: all the trade date of domestic stock market in the form of string
+# update per year by ``xa.misc.update_caldate("xalpha/caldate.csv", "2023")``
 caldate = pd.read_csv(os.path.join(__path__[0], "caldate.csv"))
+# data source
+# pro = ts.pro_api()
+# df = pro.trade_cal(exchange='', start_date='20230101', end_date='20231231')
 opendate = list(caldate[caldate["is_open"] == 1]["cal_date"])
 # opendate = list(ts.trade_cal()[ts.trade_cal()['isOpen']==1]['calendarDate'])
 opendate_set = set(opendate)  # for speed checking?
@@ -62,6 +68,8 @@ opendate_set = set(opendate)  # for speed checking?
 # fund code list which always round down for the purchase share approximation
 droplist = ["003318", "000311", "000601", "009989"]
 
+sqrt_days_in_year = sqrt(250.0)
+
 
 def calendar_selfcheck():
     # 国内链接 githubusercontent.com 大概率存在问题，因此设计成联网自动更新日历大概率无用。
@@ -96,351 +104,9 @@ region_trans = {
     "中国台湾": "TW",
 }
 
-# extract from xa.misc.get_tdx_holidays
-holidays = {
-    "AU": [
-        "2020-01-01",
-        "2020-01-27",
-        "2020-04-10",
-        "2020-04-13",
-        "2020-04-25",
-        "2020-06-08",
-        "2020-12-24",
-        "2020-12-25",
-        "2020-12-28",
-        "2020-12-31",
-        "2021-01-01",
-        "2021-01-26",
-        "2021-04-02",
-        "2021-04-05",
-        "2021-06-14",
-        "2021-12-24",
-        "2021-12-27",
-        "2021-12-28",
-        "2021-12-31",
-    ],
-    "CH": [
-        "2020-01-01",
-        "2020-01-02",
-        "2020-04-10",
-        "2020-04-13",
-        "2020-05-01",
-        "2020-05-21",
-        "2020-06-01",
-        "2020-12-24",
-        "2020-12-25",
-        "2020-12-31",
-        "2021-01-01",
-        "2021-04-02",
-        "2021-04-05",
-        "2021-05-13",
-        "2021-05-24",
-        "2021-12-24",
-        "2021-12-31",
-    ],
-    "CN": [
-        "2020-01-01",
-        "2020-01-24",
-        "2020-01-27",
-        "2020-01-28",
-        "2020-01-29",
-        "2020-01-30",
-        "2020-01-31",
-        "2020-04-06",
-        "2020-05-01",
-        "2020-05-04",
-        "2020-05-05",
-        "2020-06-25",
-        "2020-06-26",
-        "2020-10-01",
-        "2020-10-02",
-        "2020-10-05",
-        "2020-10-06",
-        "2020-10-07",
-        "2020-10-08",
-        "2021-01-01",
-        "2021-02-11",
-        "2021-02-12",
-        "2021-02-15",
-        "2021-02-16",
-        "2021-02-17",
-        "2021-04-05",
-        "2021-05-03",
-        "2021-05-04",
-        "2021-05-05",
-        "2021-06-14",
-        "2021-09-20",
-        "2021-09-21",
-        "2021-10-01",
-        "2021-10-04",
-        "2021-10-05",
-        "2021-10-06",
-        "2021-10-07",
-    ],
-    "DE": [
-        "2020-01-01",
-        "2020-04-10",
-        "2020-04-13",
-        "2020-05-01",
-        "2020-06-01",
-        "2020-12-24",
-        "2020-12-25",
-        "2020-12-31",
-        "2021-01-01",
-        "2021-04-02",
-        "2021-04-05",
-        "2021-05-24",
-        "2021-12-24",
-        "2021-12-31",
-    ],
-    "FR": [
-        "2020-01-01",
-        "2020-04-10",
-        "2020-04-13",
-        "2020-05-01",
-        "2020-12-24",
-        "2020-12-25",
-        "2020-12-31",
-        "2021-01-01",
-        "2021-04-02",
-        "2021-04-05",
-        "2021-12-24",
-        "2021-12-31",
-    ],
-    "HK": [
-        "2020-01-01",
-        "2020-01-27",
-        "2020-01-28",
-        "2020-04-10",
-        "2020-04-13",
-        "2020-04-30",
-        "2020-05-01",
-        "2020-06-25",
-        "2020-07-01",
-        "2020-10-01",
-        "2020-10-02",
-        "2020-10-26",
-        "2020-12-25",
-        "2021-01-01",
-        "2021-02-11",
-        "2021-02-12",
-        "2021-02-15",
-        "2021-04-02",
-        "2021-04-05",
-        "2021-04-06",
-        "2021-05-19",
-        "2021-06-14",
-        "2021-07-01",
-        "2021-09-22",
-        "2021-10-01",
-        "2021-10-14",
-        "2021-12-24",
-        "2021-12-27",
-        "2021-12-31",
-    ],
-    "IN": [
-        "2020-02-21",
-        "2020-03-10",
-        "2020-04-02",
-        "2020-04-06",
-        "2020-04-10",
-        "2020-04-14",
-        "2020-05-01",
-        "2020-05-25",
-        "2020-10-02",
-        "2020-11-16",
-        "2020-11-30",
-        "2020-12-25",
-        "2021-01-26",
-        "2021-03-11",
-        "2021-03-29",
-        "2021-04-02",
-        "2021-04-14",
-        "2021-04-21",
-        "2021-05-13",
-        "2021-07-20",
-        "2021-08-19",
-        "2021-09-10",
-        "2021-10-15",
-        "2021-11-04",
-        "2021-11-19",
-    ],
-    "JP": [
-        "2020-01-01",
-        "2020-01-02",
-        "2020-01-03",
-        "2020-01-13",
-        "2020-02-11",
-        "2020-02-24",
-        "2020-03-20",
-        "2020-04-29",
-        "2020-05-04",
-        "2020-05-05",
-        "2020-05-06",
-        "2020-07-23",
-        "2020-07-24",
-        "2020-08-10",
-        "2020-09-21",
-        "2020-09-22",
-        "2020-11-03",
-        "2020-11-23",
-        "2020-12-31",
-        "2021-01-01",
-        "2021-01-11",
-        "2021-02-11",
-        "2021-02-23",
-        "2021-04-29",
-        "2021-05-03",
-        "2021-05-04",
-        "2021-05-05",
-        "2021-07-22",
-        "2021-07-23",
-        "2021-08-09",
-        "2021-09-20",
-        "2021-09-23",
-        "2021-11-03",
-        "2021-11-23",
-        "2021-12-31",
-    ],
-    "KR": [
-        "2020-01-01",
-        "2020-01-24",
-        "2020-01-27",
-        "2020-04-30",
-        "2020-05-01",
-        "2020-05-05",
-        "2020-09-30",
-        "2020-10-01",
-        "2020-10-02",
-        "2020-10-09",
-        "2020-12-25",
-        "2020-12-31",
-        "2021-01-01",
-        "2021-02-11",
-        "2021-02-12",
-        "2021-03-01",
-        "2021-05-05",
-        "2021-05-19",
-        "2021-09-20",
-        "2021-09-21",
-        "2021-09-22",
-        "2021-12-31",
-    ],
-    "SG": [
-        "2020-01-01",
-        "2020-01-24",
-        "2020-04-10",
-        "2020-05-01",
-        "2020-05-07",
-        "2020-05-21",
-        "2020-07-31",
-        "2020-08-10",
-        "2020-12-24",
-        "2020-12-25",
-        "2020-12-31",
-        "2021-01-01",
-        "2021-02-11",
-        "2021-02-12",
-        "2021-04-02",
-        "2021-05-13",
-        "2021-05-26",
-        "2021-07-20",
-        "2021-08-09",
-        "2021-11-04",
-        "2021-12-24",
-        "2021-12-31",
-    ],
-    "TW": [
-        "2020-01-01",
-        "2020-01-21",
-        "2020-01-22",
-        "2020-01-23",
-        "2020-01-24",
-        "2020-01-27",
-        "2020-01-28",
-        "2020-01-29",
-        "2020-02-28",
-        "2020-04-02",
-        "2020-04-03",
-        "2020-05-01",
-        "2020-06-25",
-        "2020-06-26",
-        "2020-10-01",
-        "2020-10-02",
-        "2020-10-09",
-        "2021-01-01",
-        "2021-02-08",
-        "2021-02-09",
-        "2021-02-10",
-        "2021-02-11",
-        "2021-02-12",
-        "2021-02-15",
-        "2021-02-16",
-        "2021-03-01",
-        "2021-04-02",
-        "2021-04-05",
-        "2021-04-30",
-        "2021-06-14",
-        "2021-09-20",
-        "2021-09-21",
-        "2021-10-11",
-        "2021-12-31",
-    ],
-    "UK": [
-        "2020-01-01",
-        "2020-04-10",
-        "2020-04-13",
-        "2020-05-08",
-        "2020-05-25",
-        "2020-08-31",
-        "2020-12-24",
-        "2020-12-25",
-        "2020-12-28",
-        "2020-12-31",
-        "2021-01-01",
-        "2021-01-01",
-        "2021-04-02",
-        "2021-04-05",
-        "2021-05-03",
-        "2021-05-31",
-        "2021-08-30",
-        "2021-12-24",
-        "2021-12-27",
-        "2021-12-28",
-        "2021-12-31",
-        "2022-01-03",
-    ],
-    "US": [
-        "2020-01-01",
-        "2020-01-20",
-        "2020-02-17",
-        "2020-03-08",
-        "2020-04-10",
-        "2020-05-25",
-        "2020-07-03",
-        "2020-09-07",
-        "2020-11-01",
-        "2020-11-26",
-        "2020-11-27",
-        "2020-12-24",
-        "2020-12-25",
-        "2021-01-01",
-        "2021-01-01",
-        "2021-01-18",
-        "2021-02-15",
-        "2021-03-14",
-        "2021-04-02",
-        "2021-05-31",
-        "2021-07-05",
-        "2021-09-06",
-        "2021-11-07",
-        "2021-11-25",
-        "2021-11-26",
-        "2021-12-24",
-    ],
-}
 
 connection_errors = (
+    HttpStatusError,
     ConnectionResetError,
     requests.exceptions.RequestException,
     requests.exceptions.ConnectionError,
@@ -567,11 +233,32 @@ def next_onday(dtobj):
 def last_onday(dtobj):
     dtobj = _date_check(dtobj, check=True)
     dtobj -= dt.timedelta(1)
-    while dtobj.strftime("%Y-%m-%d") not in opendate:
+    while dtobj.strftime("%Y-%m-%d") not in opendate_set:
         dtobj -= dt.timedelta(1)
     return dtobj
 
 
+def avail_dates(dtlist, future=False):
+    """
+    make every day in the list the next open day
+
+    :param dtlist: datetime obj list
+    :param future: bool, default False, indicating the latest day in the list is yesterday
+    :return: datetime obj list
+    """
+    ndtlist = []
+    for d in dtlist:
+        if d.strftime("%Y-%m-%d") not in opendate_set:
+            nd = next_onday(d)
+        else:
+            nd = d
+        if future is False:
+            if (nd - yesterdayobj()).days > 0:
+                continue
+        ndtlist.append(nd)
+    return ndtlist
+
+
 def scale_dict(d, scale=1, ulimit=100, dlimit=50, aim=None):
     t = sum([v for _, v in d.items()])
     if t * scale > ulimit:
@@ -635,6 +322,10 @@ def reconnect(tries=5, timeout=12):
                         % (url, inspect.stack()[1].function)
                     )
                     r = f(*args, **kws)
+                    if (
+                        getattr(r, "status_code", 200) != 200
+                    ):  # in case r is a json dict
+                        raise HttpStatusError
                     return r
                 except connection_errors as e:
                     logger.warning("Fails at fetching url: %s. Try again." % url)
@@ -723,6 +414,524 @@ def rpost_json(*args, **kws):
 #                 raise e
 #             time.sleep(0.5*count)
 
+# extract from xa.misc.get_tdx_holidays
+holidays = {
+    "AU": [
+        "2020-01-01",
+        "2020-01-27",
+        "2020-04-10",
+        "2020-04-13",
+        "2020-04-25",
+        "2020-06-08",
+        "2020-12-24",
+        "2020-12-25",
+        "2020-12-28",
+        "2020-12-31",
+        "2021-01-01",
+        "2021-01-26",
+        "2021-04-02",
+        "2021-04-05",
+        "2021-06-14",
+        "2021-12-24",
+        "2021-12-27",
+        "2021-12-28",
+        "2021-12-31",
+        "2022-01-03",
+        "2022-01-26",
+        "2022-04-15",
+        "2022-04-18",
+        "2022-04-25",
+        "2022-06-13",
+        "2022-12-26",
+        "2022-12-27",
+    ],
+    "CH": [
+        "2020-01-01",
+        "2020-01-02",
+        "2020-04-10",
+        "2020-04-13",
+        "2020-05-01",
+        "2020-05-21",
+        "2020-06-01",
+        "2020-12-24",
+        "2020-12-25",
+        "2020-12-31",
+        "2021-01-01",
+        "2021-04-02",
+        "2021-04-05",
+        "2021-05-13",
+        "2021-05-24",
+        "2021-12-24",
+        "2021-12-31",
+        "2022-04-15",
+        "2022-04-18",
+        "2022-05-26",
+        "2022-06-06",
+        "2022-08-01",
+        "2022-12-26",
+    ],
+    "CN": [
+        "2020-01-01",
+        "2020-01-24",
+        "2020-01-27",
+        "2020-01-28",
+        "2020-01-29",
+        "2020-01-30",
+        "2020-01-31",
+        "2020-04-06",
+        "2020-05-01",
+        "2020-05-04",
+        "2020-05-05",
+        "2020-06-25",
+        "2020-06-26",
+        "2020-10-01",
+        "2020-10-02",
+        "2020-10-05",
+        "2020-10-06",
+        "2020-10-07",
+        "2020-10-08",
+        "2021-01-01",
+        "2021-02-11",
+        "2021-02-12",
+        "2021-02-15",
+        "2021-02-16",
+        "2021-02-17",
+        "2021-04-05",
+        "2021-05-03",
+        "2021-05-04",
+        "2021-05-05",
+        "2021-06-14",
+        "2021-09-20",
+        "2021-09-21",
+        "2021-10-01",
+        "2021-10-04",
+        "2021-10-05",
+        "2021-10-06",
+        "2021-10-07",
+        "2022-01-03",
+        "2022-01-31",
+        "2022-02-01",
+        "2022-02-02",
+        "2022-02-03",
+        "2022-02-04",
+        "2022-04-04",
+        "2022-04-05",
+        "2022-05-02",
+        "2022-05-03",
+        "2022-05-04",
+        "2022-06-03",
+        "2022-09-12",
+        "2022-10-03",
+        "2022-10-04",
+        "2022-10-05",
+        "2022-10-06",
+        "2022-10-07",
+        "2023-01-02",
+        "2023-01-23",
+        "2023-01-24",
+        "2023-01-25",
+        "2023-01-26",
+        "2023-01-27",
+        "2023-04-05",
+        "2023-05-01",
+        "2023-05-02",
+        "2023-05-03",
+        "2023-06-22",
+        "2023-06-23",
+        "2023-09-29",
+        "2023-10-02",
+        "2023-10-03",
+        "2023-10-04",
+        "2023-10-05",
+        "2023-10-06",
+    ],
+    "DE": [
+        "2020-01-01",
+        "2020-04-10",
+        "2020-04-13",
+        "2020-05-01",
+        "2020-06-01",
+        "2020-12-24",
+        "2020-12-25",
+        "2020-12-31",
+        "2021-01-01",
+        "2021-04-02",
+        "2021-04-05",
+        "2021-05-24",
+        "2021-12-24",
+        "2021-12-31",
+        "2022-04-15",
+        "2022-04-18",
+        "2022-12-26",
+    ],
+    "FR": [
+        "2020-01-01",
+        "2020-04-10",
+        "2020-04-13",
+        "2020-05-01",
+        "2020-12-24",
+        "2020-12-25",
+        "2020-12-31",
+        "2021-01-01",
+        "2021-04-02",
+        "2021-04-05",
+        "2021-12-24",
+        "2021-12-31",
+        "2022-04-15",
+        "2022-04-18",
+        "2022-12-26",
+    ],
+    "HK": [
+        "2020-01-01",
+        "2020-01-27",
+        "2020-01-28",
+        "2020-04-10",
+        "2020-04-13",
+        "2020-04-30",
+        "2020-05-01",
+        "2020-06-25",
+        "2020-07-01",
+        "2020-10-01",
+        "2020-10-02",
+        "2020-10-26",
+        "2020-12-25",
+        "2021-01-01",
+        "2021-02-11",
+        "2021-02-12",
+        "2021-02-15",
+        "2021-04-02",
+        "2021-04-05",
+        "2021-04-06",
+        "2021-05-19",
+        "2021-06-14",
+        "2021-07-01",
+        "2021-09-22",
+        "2021-10-01",
+        "2021-10-14",
+        "2021-12-24",
+        "2021-12-27",
+        "2021-12-31",
+        "2022-01-31",
+        "2022-02-01",
+        "2022-02-02",
+        "2022-02-03",
+        "2022-04-05",
+        "2022-04-15",
+        "2022-04-18",
+        "2022-05-02",
+        "2022-05-09",
+        "2022-06-03",
+        "2022-07-01",
+        "2022-09-12",
+        "2022-10-04",
+        "2022-12-26",
+        "2022-12-27",
+        "2023-01-02",
+        "2023-01-23",
+        "2023-01-24",
+        "2023-01-25",
+        "2023-04-05",
+        "2023-04-07",
+        "2023-04-10",
+        "2023-05-01",
+        "2023-05-26",
+        "2023-06-22",
+        "2023-10-02",
+        "2023-10-23",
+        "2023-12-25",
+        "2023-12-26",
+    ],
+    "IN": [
+        "2020-02-21",
+        "2020-03-10",
+        "2020-04-02",
+        "2020-04-06",
+        "2020-04-10",
+        "2020-04-14",
+        "2020-05-01",
+        "2020-05-25",
+        "2020-10-02",
+        "2020-11-16",
+        "2020-11-30",
+        "2020-12-25",
+        "2021-01-26",
+        "2021-03-11",
+        "2021-03-29",
+        "2021-04-02",
+        "2021-04-14",
+        "2021-04-21",
+        "2021-05-13",
+        "2021-07-20",
+        "2021-08-19",
+        "2021-09-10",
+        "2021-10-15",
+        "2021-11-04",
+        "2021-11-19",
+        "2022-01-26",
+        "2022-03-01",
+        "2022-03-18",
+        "2022-04-14",
+        "2022-04-15",
+        "2022-05-02",
+        "2022-08-09",
+        "2022-08-15",
+        "2022-08-31",
+        "2022-10-04",
+        "2022-10-25",
+        "2022-11-07",
+    ],
+    "JP": [
+        "2020-01-01",
+        "2020-01-02",
+        "2020-01-03",
+        "2020-01-13",
+        "2020-02-11",
+        "2020-02-24",
+        "2020-03-20",
+        "2020-04-29",
+        "2020-05-04",
+        "2020-05-05",
+        "2020-05-06",
+        "2020-07-23",
+        "2020-07-24",
+        "2020-08-10",
+        "2020-09-21",
+        "2020-09-22",
+        "2020-11-03",
+        "2020-11-23",
+        "2020-12-31",
+        "2021-01-01",
+        "2021-01-11",
+        "2021-02-11",
+        "2021-02-23",
+        "2021-04-29",
+        "2021-05-03",
+        "2021-05-04",
+        "2021-05-05",
+        "2021-07-22",
+        "2021-07-23",
+        "2021-08-09",
+        "2021-09-20",
+        "2021-09-23",
+        "2021-11-03",
+        "2021-11-23",
+        "2021-12-31",
+        "2022-01-03",
+        "2022-01-10",
+        "2022-02-11",
+        "2022-02-23",
+        "2022-03-21",
+        "2022-04-29",
+        "2022-05-03",
+        "2022-05-04",
+        "2022-05-05",
+        "2022-07-18",
+        "2022-08-11",
+        "2022-09-19",
+        "2022-09-23",
+        "2022-10-10",
+        "2022-11-03",
+        "2022-11-23",
+    ],
+    "KR": [
+        "2020-01-01",
+        "2020-01-24",
+        "2020-01-27",
+        "2020-04-30",
+        "2020-05-01",
+        "2020-05-05",
+        "2020-09-30",
+        "2020-10-01",
+        "2020-10-02",
+        "2020-10-09",
+        "2020-12-25",
+        "2020-12-31",
+        "2021-01-01",
+        "2021-02-11",
+        "2021-02-12",
+        "2021-03-01",
+        "2021-05-05",
+        "2021-05-19",
+        "2021-09-20",
+        "2021-09-21",
+        "2021-09-22",
+        "2021-12-31",
+        "2022-01-31",
+        "2022-02-01",
+        "2022-02-02",
+        "2022-03-01",
+        "2022-05-05",
+        "2022-06-06",
+        "2022-08-15",
+        "2022-09-09",
+        "2022-09-12",
+        "2022-10-03",
+        "2022-10-10",
+        "2022-12-30",
+    ],
+    "SG": [
+        "2020-01-01",
+        "2020-01-24",
+        "2020-04-10",
+        "2020-05-01",
+        "2020-05-07",
+        "2020-05-21",
+        "2020-07-31",
+        "2020-08-10",
+        "2020-12-24",
+        "2020-12-25",
+        "2020-12-31",
+        "2021-01-01",
+        "2021-02-11",
+        "2021-02-12",
+        "2021-04-02",
+        "2021-05-13",
+        "2021-05-26",
+        "2021-07-20",
+        "2021-08-09",
+        "2021-11-04",
+        "2021-12-24",
+        "2021-12-31",
+        "2022-01-31",
+        "2022-02-01",
+        "2022-02-02",
+        "2022-04-15",
+        "2022-05-02",
+        "2022-05-03",
+        "2022-05-16",
+        "2022-07-11",
+        "2022-08-09",
+        "2022-10-24",
+        "2022-12-26",
+    ],
+    "TW": [
+        "2020-01-01",
+        "2020-01-21",
+        "2020-01-22",
+        "2020-01-23",
+        "2020-01-24",
+        "2020-01-27",
+        "2020-01-28",
+        "2020-01-29",
+        "2020-02-28",
+        "2020-04-02",
+        "2020-04-03",
+        "2020-05-01",
+        "2020-06-25",
+        "2020-06-26",
+        "2020-10-01",
+        "2020-10-02",
+        "2020-10-09",
+        "2021-01-01",
+        "2021-02-08",
+        "2021-02-09",
+        "2021-02-10",
+        "2021-02-11",
+        "2021-02-12",
+        "2021-02-15",
+        "2021-02-16",
+        "2021-03-01",
+        "2021-04-02",
+        "2021-04-05",
+        "2021-04-30",
+        "2021-06-14",
+        "2021-09-20",
+        "2021-09-21",
+        "2021-10-11",
+        "2021-12-31",
+        "2022-01-27",
+        "2022-01-28",
+        "2022-01-31",
+        "2022-02-01",
+        "2022-02-02",
+        "2022-02-03",
+        "2022-02-04",
+        "2022-02-28",
+        "2022-04-04",
+        "2022-04-05",
+        "2022-05-02",
+        "2022-06-03",
+        "2022-09-09",
+        "2022-10-10",
+    ],
+    "UK": [
+        "2020-01-01",
+        "2020-04-10",
+        "2020-04-13",
+        "2020-05-08",
+        "2020-05-25",
+        "2020-08-31",
+        "2020-12-24",
+        "2020-12-25",
+        "2020-12-28",
+        "2020-12-31",
+        "2021-01-01",
+        "2021-01-01",
+        "2021-04-02",
+        "2021-04-05",
+        "2021-05-03",
+        "2021-05-31",
+        "2021-08-30",
+        "2021-12-24",
+        "2021-12-27",
+        "2021-12-28",
+        "2021-12-31",
+        "2022-01-03",
+        "2022-04-15",
+        "2022-04-18",
+        "2022-05-02",
+        "2022-06-02",
+        "2022-06-03",
+        "2022-08-29",
+        "2022-12-23",
+        "2022-12-26",
+        "2022-12-27",
+        "2022-12-30",
+        "2023-01-02",
+        "2022-09-19",
+    ],
+    "US": [
+        "2020-01-01",
+        "2020-01-20",
+        "2020-02-17",
+        "2020-03-08",
+        "2020-04-10",
+        "2020-05-25",
+        "2020-07-03",
+        "2020-09-07",
+        "2020-11-01",
+        "2020-11-26",
+        "2020-11-27",
+        "2020-12-24",
+        "2020-12-25",
+        "2021-01-01",
+        "2021-01-01",
+        "2021-01-18",
+        "2021-02-15",
+        "2021-03-14",
+        "2021-04-02",
+        "2021-05-31",
+        "2021-07-05",
+        "2021-09-06",
+        "2021-11-07",
+        "2021-11-25",
+        "2021-11-26",
+        "2021-12-24",
+        "2022-01-17",
+        "2022-02-21",
+        "2022-03-13",
+        "2022-04-15",
+        "2022-05-30",
+        "2022-06-20",
+        "2022-07-04",
+        "2022-09-05",
+        "2022-11-06",
+        "2022-11-24",
+        "2022-11-25",
+        "2022-12-26",
+    ],
+}
+
 
 ## simple subsitution for holdings.py
 
