@@ -119,13 +119,13 @@ class mul:
                             status,
                         )
                     )
-            if istatus is not None:
-                self.is_in = True
-                if isinstance(istatus, irecord):
-                    istatus = istatus.status
-                for code in istatus.code.unique():
-                    if code not in fundcodelist and not code.startswith("#"):
-                        fundtradeobj.append(itrade(code, istatus))
+        if istatus is not None:
+            self.is_in = True
+            if isinstance(istatus, irecord):
+                istatus = istatus.status
+            for code in istatus.code.unique():
+                if code not in fundcodelist and not code.startswith("#"):
+                    fundtradeobj.append(itrade(code, istatus))
         self.fundtradeobj = tuple(fundtradeobj)
         self.totcftable = self._mergecftb()
 
@@ -139,7 +139,7 @@ class mul:
         """
         res = 0
         for fund in self.fundtradeobj:
-            res += fund.dailyreport().iloc[0][prop]
+            res += fund.dailyreport(date).iloc[0][prop]
         return res
 
     def combsummary(self, date=yesterdayobj()):
