@@ -514,6 +514,38 @@ def get_fundshare_byjq(code, **kws):
     return df
 
 
+@lru_cache(maxsize=1024)
+def get_futu_id(code):
+    r = rget("https://www.futunn.com/stock/{code}".format(code=code))
+    sind = r.text.find("securityId")
+    futuid = r.text[sind : sind + 30].split("=")[1].split(";")[0].strip(" ").strip("'")
+    sind = r.text.find("marketType")
+    market = r.text[sind : sind + 30].split("=")[1].split(";")[0].strip().strip("''")
+    return futuid, market
+
+
+def get_futu_historical(code, start=None, end=None):
+    fid, market = get_futu_id(code)
+    r = rget(
+        "https://www.futunn.com/new-quote/kline?security_id={fid}&type=2&market_type={market}".format(
+            fid=fid, market=market
+        )
+    )
+    df = pd.DataFrame(r.json()["data"]["list"])
+    df["date"] = df["k"].map(
+        lambda s: dt.datetime.fromtimestamp(s)
+        .replace(hour=0, minute=0, second=0, microsecond=0)
+        .replace(tzinfo=None)
+    )
+    df["open"] = df["o"] / 1000
+    df["close"] = df["c"] / 1000
+    df["high"] = df["h"] / 1000
+    df["low"] = df["l"] / 1000
+    df["volume"] = df["v"]
+    df = df.drop(["k", "t", "o", "c", "h", "l", "v"], axis=1)
+    return df
+
+
 def get_historical_fromsp(code, start=None, end=None, region="us", **kws):
     """
     标普官网数据源
@@ -1008,6 +1040,8 @@ def _get_daily(
 
             26. 形如 B-AA+.3 格式的数据，代表了 AA+ 企业债三年久期利率数据 (每周)
 
+            27. 形如 fu-00700.HK 或 fu-BA.US 格式的数据，代表了来自 https://www.futunn.com/stock/BA-US 的日线行情数据
+
     :param start: str. "20200101", "2020/01/01", "2020-01-01" are all legal. The starting date of daily data.
     :param end: str. format is the same as start. The ending date of daily data.
     :param prev: Optional[int], default 365. If start is not specified, start = end-prev.
@@ -1143,6 +1177,10 @@ def _get_daily(
     elif _from == "B":
         df = get_bond_rates_range(code, start=start, end=end)
 
+    elif _from == "fu":
+        code = code.replace(".", "-")
+        df = get_futu_historical(code, start=start, end=end)
+
     elif _from == "ycharts":
         df = get_historical_fromycharts(
             code,
@@ -1588,27 +1626,32 @@ def get_rt_from_ttjj(code):
             ),
             str(s.findAll("dt")[1]).split("(")[1].split(")")[0][7:],
         )
-        estimate = s.select("span[id=gz_gsz]")[0].text  # after loading
-        if estimate == "--":
-            gsz = rget(
-                "http://fundgz.1234567.com.cn/js/{code}.js".format(code=code),
-                headers={
-                    "Host": "fundgz.1234567.com.cn",
-                    "Referer": "http://fund.eastmoney.com/",
-                },
-            )
-            try:  # in case eval error
-                gsz_dict = eval(gsz.text[8:-2])
-                estimate = _float(gsz_dict["gsz"])
-                estimate_time = gsz_dict["gztime"]
-            except:
-                estimate = None
+        estimate = s.select("span[id=gz_gsz]")
+        if not estimate:
+            estimate = None
+            estimate_time = None
         else:
-            try:
-                estimate = _float(estimate)
-            except ValueError:
-                logger.warning("unrecognized estimate netvalue %s" % estimate)
-                estimate = None
+            estimate = estimate[0].text  # after loading
+            if estimate == "--":
+                gsz = rget(
+                    "http://fundgz.1234567.com.cn/js/{code}.js".format(code=code),
+                    headers={
+                        "Host": "fundgz.1234567.com.cn",
+                        "Referer": "http://fund.eastmoney.com/",
+                    },
+                )
+                try:  # in case eval error
+                    gsz_dict = eval(gsz.text[8:-2])
+                    estimate = _float(gsz_dict["gsz"])
+                    estimate_time = gsz_dict["gztime"]
+                except:
+                    estimate = None
+            else:
+                try:
+                    estimate = _float(estimate)
+                except ValueError:
+                    logger.warning("unrecognized estimate netvalue %s" % estimate)
+                    estimate = None
     else:
         value, date = (
             s.findAll("dd", class_="dataNums")[1].text,
@@ -2331,8 +2374,8 @@ def get_peb(index, date=None, table=False):
     ).replace(day=1)
     iwdf = get_index_weight_range(
         index,
-        start=(middle - dt.timedelta(days=5)).strftime("%Y-%m-%d"),
-        end=(middle + dt.timedelta(days=5)).strftime("%Y-%m-%d"),
+        start=(middle - dt.timedelta(days=10)).strftime("%Y-%m-%d"),
+        end=(middle + dt.timedelta(days=6)).strftime("%Y-%m-%d"),
     )
     q = query(valuation).filter(valuation.code.in_(list(iwdf.code)))
     logger.debug("get_fundamentals on %s" % (date))
@@ -2345,7 +2388,10 @@ def get_peb(index, date=None, table=False):
     totb = df.b.sum()
     if table:
         return df
-    return {"pe": round(100.0 / tote, 3), "pb": round(100.0 / totb, 3)}
+    return {
+        "pe": (round(100.0 / tote, 3) if tote != 0 else np.inf),
+        "pb": (round(100.0 / totb, 3) if totb != 0 else np.inf),
+    }
 
 
 @data_source("jq")
